@@ -1,5 +1,5 @@
 ;; comm.scm -- Communication between processes and general output.
-;; Copyright (C) 2013, 2014, 2016, 2018, 2019, 2022 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2014, 2016, 2018, 2019, 2022, 2023 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;; Copyright (C) 2018 Danny Milosavljevic <dannym@scratchpost.org>
 ;;
@@ -22,13 +22,14 @@
   #:use-module (shepherd support)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9)
-  #:use-module (oop goops)
   #:use-module (ice-9 match)
   #:export (open-connection
+            open-server-socket
 
             <shepherd-command>
             shepherd-command?
             shepherd-command
+            shepherd-command-version
             shepherd-command-directory
             shepherd-command-action
             shepherd-command-service
@@ -46,32 +47,39 @@
             read-command
 
             write-reply
+            define-record-type-serializer
             result->sexp
             report-command-error
 
             log-output-port
             syslog-output-port
-            start-logging
-            stop-logging
             make-shepherd-output-port
 
             %current-client-socket
-            %current-logfile-date-format))
+            %current-logfile-date-format
+            %current-service-output-port))
 
 
 ;; Command for shepherd.
 (define-record-type <shepherd-command>
-  (%shepherd-command action service args directory)
+  (%shepherd-command version action service args directory)
   shepherd-command?
+  (version   shepherd-command-version)            ; list of integers
   (action    shepherd-command-action)             ; symbol
   (service   shepherd-command-service)            ; symbol
   (args      shepherd-command-arguments)          ; list of strings
   (directory shepherd-command-directory))         ; directory name
 
+(define %protocol-version
+  ;; Current client protocol version.
+  '(0))
+
 (define* (shepherd-command action service
-                           #:key (arguments '()) (directory (getcwd)))
+                           #:key
+                           (version %protocol-version)
+                           (arguments '()) (directory (getcwd)))
   "Return a new command for ACTION on SERVICE."
-  (%shepherd-command action service arguments directory))
+  (%shepherd-command version action service arguments directory))
 
 (define* (open-connection #:optional (file default-socket-file))
   "Open a connection to the daemon, using the Unix-domain socket at FILE, and
@@ -93,20 +101,36 @@ return the socket."
                  (list errno) rest)))
       sock)))
 
+(define (open-server-socket file-name)
+  "Open a socket at FILE-NAME, and listen for connections there."
+  (with-fluids ((%default-port-encoding "UTF-8"))
+    (let ((sock    (socket PF_UNIX
+                           (logior SOCK_STREAM SOCK_NONBLOCK SOCK_CLOEXEC)
+                           0))
+          (address (make-socket-address AF_UNIX file-name)))
+      (bind sock address)
+      (listen sock 10)
+      sock)))
+
 (define (read-command port)
   "Receive a command from PORT; return the command of #f if something went
 wrong---premature end-of-file, invalid sexp, etc."
   (catch 'read-error
     (lambda ()
       (match (read port)
-        (('shepherd-command ('version 0 _ ...)
+        (('shepherd-command ('version version ...)
                             ('action action)
                             ('service service)
                             ('arguments (args ...))
                             ('directory directory))
-         (shepherd-command action service
-                           #:arguments args
-                           #:directory directory))
+         (match version
+           ((0 _ ...)
+            (shepherd-command action service
+                              #:version version
+                              #:arguments args
+                              #:directory directory))
+           (_
+            #f)))
         (_                                        ;EOF or unmatched sexp
          #f)))
     (lambda _
@@ -116,8 +140,9 @@ wrong---premature end-of-file, invalid sexp, etc."
 (define (write-command command port)
   "Write COMMAND to PORT."
   (match command
-    (($ <shepherd-command> action service (arguments ...) directory)
-     (write `(shepherd-command (version 0)        ; protocol version
+    (($ <shepherd-command> version
+        action service (arguments ...) directory)
+     (write `(shepherd-command (version ,@version) ;protocol version
                                (action ,action)
                                (service ,service)
                                (arguments ,arguments)
@@ -154,19 +179,37 @@ wrong---premature end-of-file, invalid sexp, etc."
      ;; PORT may be buffered so make sure the command goes out.
      (force-output port))))
 
-;; This generic function must be extended to provide sexp representations of
-;; results that go in <command-reply> objects.
-(define-generic result->sexp)
+(define %record-serializers
+  ;; Hash table mapping record type descriptors (RTDs) to procedures that
+  ;; "convert" instances to an sexp.
+  (make-hash-table 3))
 
-(define-method (result->sexp (bool <boolean>)) bool)
-(define-method (result->sexp (number <number>)) number)
-(define-method (result->sexp (symbol <symbol>)) symbol)
-(define-method (result->sexp (string <string>)) string)
-(define-method (result->sexp (list <list>)) (map result->sexp list))
-(define-method (result->sexp (pair <pair>))
-  (cons (result->sexp (car pair)) (result->sexp (cdr pair))))
-(define-method (result->sexp (kw <keyword>)) kw)
-(define-method (result->sexp (obj <top>)) (object->string obj))
+(define-syntax-rule (define-record-type-serializer (name (obj type))
+                      body ...)
+  "Define @var{name} as a procedure that, given @var{obj}, a record of
+@var{type}, returns an sexp serialization of that record."
+  (hashq-set! %record-serializers type
+              (lambda (obj)
+                body ...)))
+
+(define (result->sexp obj)
+  "Return the sexp representation of @var{obj}, a result meant to go in a
+@code{<command-reply>} object."
+  (cond ((or (boolean? obj) (number? obj) (symbol? obj)
+             (string? obj) (keyword? obj))
+         obj)
+        ((list? obj)
+         (map result->sexp obj))
+        ((pair? obj)
+         (cons (result->sexp (car obj)) (result->sexp (cdr obj))))
+        ((struct? obj)
+         (let ((serializer (hashq-ref %record-serializers
+                                      (struct-vtable obj))))
+           (if serializer
+               (serializer obj)
+               (object->string obj))))
+        (else
+         (object->string obj))))
 
 (define (report-command-error error)
   "Report ERROR, an sexp received by a shepherd client in reply to COMMAND, a
@@ -202,15 +245,6 @@ on service '~a':")
   ;; Port for logging.  This must always be a valid port, never `#f'.
   (make-parameter (%make-void-port "w")))
 
-(define (start-logging file)                      ;deprecated
-  (let ((directory (dirname file)))
-    (unless (file-exists? directory)
-      (mkdir directory)))
-  (log-output-port (open-file file "al")))
-(define (stop-logging)                            ;deprecated
-  (close-port (log-output-port))
-  (log-output-port (%make-void-port "w")))
-
 (define %current-client-socket
   ;; Socket of the client currently talking to the daemon.
   (make-parameter #f))
@@ -345,3 +379,6 @@ available."
    ;; It's an output-only port.
    "w"))
 
+(define %current-service-output-port
+  ;; The output port that services should write to.
+  (make-parameter (current-output-port)))
