@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test detecting a forked process' termination
-# Copyright © 2016, 2020, 2022 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2016, 2020, 2022, 2023 Ludovic Courtès <ludo@gnu.org>
 # Copyright © 2018 Carlo Zancanaro <carlo@zancanaro.id.au>
 #
 # This file is part of the GNU Shepherd.
@@ -53,48 +53,49 @@ cat > "$conf"<<EOF
   '("$SHELL" "-c" "ulimit -n >$PWD/$service_nofiles; sleep 600 & echo \$! > $PWD/$service_pid"))
 
 (register-services
- (make <service>
-   ;; A service that forks into a different process.
-   #:provides '(test)
-   #:start (make-forkexec-constructor %command
-                                      #:pid-file "$PWD/$service_pid"
-                                      #:resource-limits '((nofile 1567 1567)))
-   #:stop  (make-kill-destructor)
-   #:respawn? #f))
+ (list (service
+	 ;; A service that forks into a different process.
+	 '(test)
+	 #:start (make-forkexec-constructor
+                   %command
+                   #:pid-file "$PWD/$service_pid"
+                   #:resource-limits '((nofile 1567 1567)))
+	 #:stop  (make-kill-destructor)
+	 #:respawn? #f)))
 
 (define %command2
   '("$SHELL" "-c" "echo started >> $PWD/$service2_started; sleep 600 & echo \$! > $PWD/$service2_pid"))
 
 (register-services
- (make <service>
-   ;; A service that forks into a different process.
-   #:provides '(test2)
-   #:start (make-forkexec-constructor %command2
-                                      #:pid-file "$PWD/$service2_pid")
-   #:stop  (make-kill-destructor)
-   #:respawn? #t))
+ (list (service
+	 ;; A service that forks into a different process.
+	 '(test2)
+	 #:start (make-forkexec-constructor %command2
+					    #:pid-file "$PWD/$service2_pid")
+	 #:stop  (make-kill-destructor)
+	 #:respawn? #t)))
 
 (define %command3
   '("$SHELL" "-c" "sleep 600"))
 
 (register-services
- (make <service>
-   ;; A service that forks into a different process.
-   #:provides '(test3)
-   #:start (make-forkexec-constructor %command3)
-   #:stop  (make-kill-destructor)
-   #:respawn? #t))
+ (list (service
+	 ;; A service that forks into a different process.
+	 '(test3)
+	 #:start (make-forkexec-constructor %command3)
+	 #:stop  (make-kill-destructor)
+	 #:respawn? #t)))
 
 (define %command4
   '("$SHELL" "-c" "trap 'echo ignoring SIGTERM' SIGTERM; while true ; do : ; done"))
 
 (register-services
- (make <service>
-   ;; A service that ignores SIGTERM.
-   #:provides '(test4)
-   #:requires '(test3)
-   #:start (make-forkexec-constructor %command4)
-   #:stop  (make-kill-destructor SIGTERM #:grace-period 3)))
+ (list (service
+	 ;; A service that ignores SIGTERM.
+	 '(test4)
+	 #:requirement '(test3)
+	 #:start (make-forkexec-constructor %command4)
+	 #:stop  (make-kill-destructor SIGTERM #:grace-period 3))))
 EOF
 cat $conf
 
@@ -111,7 +112,7 @@ $herd start test
 $herd start test2
 
 # make sure "test" is started
-until $herd status test | grep started; do sleep 0.3; done
+until $herd status test | grep running; do sleep 0.3; done
 test -f "$service_pid"
 service_pid_value="`cat $service_pid`"
 # now kill it
@@ -124,7 +125,7 @@ $herd status test | grep stopped
 
 
 # make sure "test2" has started
-until $herd status test2 | grep started; do sleep 0.3; done
+until $herd status test2 | grep running; do sleep 0.3; done
 test -f "$service2_pid"
 service2_pid_value="`cat $service2_pid`"
 test "`cat $PWD/$service2_started`" = "started"
@@ -134,7 +135,7 @@ kill $service2_pid_value
 while kill -0 "$service2_pid_value"; do sleep 0.3; done
 # shepherd should notice that the service has stopped, and restart it, within one second
 sleep 1;
-$herd status test2 | grep started
+$herd status test2 | grep running
 test "`cat $PWD/$service2_started`" = "started
 started"
 
@@ -154,11 +155,11 @@ done
 
 # Make sure 'herd stop' eventually terminates processes that ignore SIGTERM.
 $herd start test4
-$herd status test3 | grep started
+$herd status test3 | grep running
 child_pid="$($herd status test4 | grep Running | sed '-es/.*Running value is \([0-9]\+\)\./\1/g')"
 kill -0 "$child_pid"
 $herd stop test3		# this will also stop 'test4'
-! kill -0 "$child_pid"
+kill -0 "$child_pid" && false
 grep ignoring "$log"
 grep SIGKILL "$log"
 $herd status test3 | grep stopped
