@@ -1,5 +1,5 @@
 # GNU Shepherd --- Make sure shepherd tolerates misbehaved clients.
-# Copyright © 2016, 2018 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2016, 2018, 2022 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -68,7 +68,7 @@ $herd status			# still here?
 (use-modules (shepherd comm) (shepherd support))
 
 (let ((sock (open-connection \"$socket\")))
-  (setvbuf sock (buffering-mode block) 5000)
+  (setvbuf sock 'block 5000)
   (write-command (shepherd-command 'status 'root) sock)
 
   ;; Close prematurely, right after sending the command.
@@ -76,4 +76,28 @@ $herd status			# still here?
 
 $herd status
 
+if "$GUILE" -c '(exit (string=? "2" (major-version)))'
+then
+    # The 'read' procedure in Guile 2.2 is not suspendable.  Thus, the test
+    # below would hang forever in 'scm_read'.
+    echo "Running on Guile 2.x; skipping test that requires suspendable 'read'." >&2
+else
+    "$GUILE" -c "
+(use-modules (shepherd comm) (shepherd support) (ice-9 match))
+
+(let ((sock (open-connection \"$socket\")))
+  (setvbuf sock 'none)
+  (display \"(ah ha!\" sock)
+  ;; Leave it hanging; shepherd must not block.
+  (let ((sock (open-connection \"$socket\")))
+    (write-command (shepherd-command 'status 'does-not-exist) sock)
+    (match (read sock)
+      (('reply _ ...) #t)
+      (x
+       (pk 'wrong x)
+       (exit 1)))))"
+
+    $herd status
+fi
+
 cat "$log"
