@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test one-shot services.
-# Copyright © 2019 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2019, 2023 Ludovic Courtès <ludo@gnu.org>
 #
 # This file is part of the GNU Shepherd.
 #
@@ -34,27 +34,47 @@ trap "cat $log || true; rm -f $socket $conf $stamp $log;
 cat > "$conf"<<EOF
 (use-modules (srfi srfi-26))
 (register-services
- (make <service>
-   #:provides '(always-fail)
-   #:start (const #f)
-   #:one-shot? #t)
- (make <service>
-   #:provides '(test)
-   #:start (lambda _
-             (call-with-output-file "$stamp"
-               (cut display "foo" <>))
-             #t)
-   #:one-shot? #t)
- (make <service>
-   #:provides '(test-2)
-   #:requires '(test)
-   #:start (lambda _
-             (call-with-output-file "$stamp-2"
-               (cut display "bar" <>))
-             #t)
-   #:stop  (lambda _
-             (delete-file "$stamp-2")
-             #f)))
+ (list (service
+	 '(always-fail)
+	 #:start (const #f)
+	 #:one-shot? #t)
+       (service
+	 '(test)
+	 #:start (lambda _
+		   (call-with-output-file "$stamp"
+		     (cut display "foo" <>))
+		   #t)
+	 #:one-shot? #t)
+       (service
+	 '(test-2)
+	 #:requirement '(test)
+	 #:start (lambda _
+		   (call-with-output-file "$stamp-2"
+		     (cut display "bar" <>))
+		   #t)
+	 #:stop  (lambda _
+		   (delete-file "$stamp-2")
+		   #f))
+
+       ;; Several services depending on the same one-shot service.
+       (service
+	 '(one-shotty)
+	 #:start (const #t)
+	 #:one-shot? #t)
+       (service
+	 '(a)
+	 #:requirement '(one-shotty)
+	 #:start (const #t))
+       (service
+	 '(b)
+	 #:requirement '(a one-shotty)
+	 #:start (const #t))
+       (service
+	 '(c)
+	 #:requirement '(a b one-shotty)
+	 #:start (const #t))))
+
+(start-in-the-background '(a b c))
 EOF
 
 rm -f "$pid"
@@ -89,6 +109,24 @@ $herd start test-2
 test -f "$stamp"
 test -f "$stamp-2"
 $herd status test | grep stopped.*one-shot
-$herd status test-2 | grep started
+$herd status test-2 | grep running
 $herd stop test-2
 if test -f "$stamp-2"; then false; else true; fi
+
+# When starting A, B, and C via 'start-in-the-background', ONE-SHOTTY should
+# have been started once only.
+test $(grep "Starting service one-shotty" "$log" | wc -l) -eq 1
+$herd stop a
+
+# In the course of starting C, ONE-SHOTTY should be started only once.
+$herd start c
+test $(grep "Starting service one-shotty" "$log" | wc -l) -eq 2
+
+# But we can still start it a second time, indirectly...
+$herd stop a
+$herd start c
+test $(grep "Starting service one-shotty" "$log" | wc -l) -eq 3
+
+# ... and a third time, directly.
+$herd start one-shotty
+test $(grep "Starting service one-shotty" "$log" | wc -l) -eq 4
